#!/usr/bin/env python
#1 wire temp tests
						  #+---------+
						  #|  front  |
						  #+-+--+--+-+
							#|  |  |
							#1  2  3

	#Where::
		#1 = ground (port 9, pin 1 or 2)
		#2 = data   (port 8, pin 6)
		#3 = Vcc	(port 9, pin 3 or 4)
	#Once you have it connected you can read the temperature from the following
	#file::
		#$ cat /sys/bus/w1/devices/<unique id>/w1_slave
  
from bbio import *
from SafeProcess import *
import os.path

probes = {}

def setup():
	print "startup"
	probes["probe1"] = Thermo("28-00000400c49b")
	probes["probe2"] = Thermo("28-00000463288d")
	probes["probe3"] = Thermo("28-00000400a75e")
	p = SafeProcess(target=temploop)
	p.start()

def loop():
	# And a little delay to keep the Beaglebone happy:
	delay(2000)
	for k, v in enumerate(probes):
		print "{0} : {1} : {2}".format(time.strftime('%x %X'), v, probes[v].tempC)
	print ""

def temploop():
#	global probes
	while(True):
		for k, v in enumerate(probes):
			probes[v].read()
		delay(1000)
	
class Thermo():
	_read_path = '/sys/bus/w1/devices/{0}/w1_slave'
	_uid = ''
	_value = 0.0
	_buf = None

	def __init__(self, uid):
		self._uid = uid
	
	@property
	def tempC(self):
		return self._value
	
	@property
	def buf(self):
		if self._buf is None:
			self._buf = open(self._read_path.format(self._uid, 'r'))
		return self._buf
	
	def _get_value(self):
		s = self.buf.read()
		i = s.find('t=')
		self.buf.seek(0)
		return float(s[i+2:]) / 1000.0

	def read(self):
		try:
			value = self._get_value()
		except ValueError:
			pass
		else:
			difference = abs(self._value - value)
			if value is not None and value >= 0 and difference > 0: #sometimes the dallas thermometers seem to crap out and show an invalid negative value
				self._value = value
	
	def do_shutdown(self):
		self.buf.close()
		
run(setup, loop)
